#ifndef AABB_H_
#define AABB_H_

#include "vector3.h"

/*! \class AABB
\brief Obalov struktura.

Obalov struktura v podob osov zarovnanho kvdru (axis aligned bounding
box), stny kvdru jsou kolm na souadn osy. Rozsah obalov struktury je
uren dvojic vektor pedstavujc doln a horn mez.

\author Tom Fabin
\version 1.0
\date 2011-2012
*/
class AABB
{
public:
	//! Implicitn konstruktor.
	/*!
	Inicializuje meze struktury na
	hodnoty \f$(+\infty,+\infty,+\infty)\f$ a \f$(-\infty,-\infty,-\infty)\f$,
	tj. struktura nic neobsahuje.
	*/
	AABB();

	//! Specializovan konstruktor.
	/*!
	Inicializuje meze struktury na hodnoty \a p0 a \a p1.

	\param p0 spodn mez.
	\param p1 horn mez.
	*/
	AABB( const Vector3 & p0, const Vector3 & p1 );

	//! Slouen dvou obalovch struktur.
	/*!
	Nov meze struktury budou obsahovat zadanou \a aabb strukturu.

	\param aabb druh obalov struktura.
	*/
	void Merge( const AABB & aabb );

	//! Zahrnut bodu do obalov struktury.
	/*!
	Obalov struktura bude rozena tak, aby obsahovala zadan bod \a p.

	\param p bod.
	*/
	void Merge( const Vector3 & p );

	//! Index dominantn osy.
	/*!	
	\return Index dominantn osy.
	*/
	char dominant_axis() const;

	//! Vypote geometrick sted obalov struktury.
	/*!	
	\return Sted obalov struktury.
	*/
	Vector3 center() const;

	//! Vypote plochu stn obalov struktury.
	/*!	
	\return Plocha stn obalov struktury.
	*/
	float surface_area() const;

	//! Doln mez obalov struktury.
	/*!	
	\return Doln mez obalov struktury.
	*/
	Vector3 lower_bound() const;

	//! Horn mez obalov struktury.
	/*!	
	\return Horn mez obalov struktury.
	*/
	Vector3 upper_bound() const;	

	Vector3 & operator[]( int i )
	{
		return bounds_[i];
	}

private:
	Vector3 bounds_[2]; /*!< Doln [0] a horn [1] mez obalov struktury. */
};

#endif
