#ifndef VOLUME_H_
#define VOLUME_H_

#include "vector3.h"
#include "matrix4x4.h"
#include "aabb.h"
#include "ray.h"
#include "intersection.h"
#include "cell.h"
#include "camera.h"

float Random( const float range_min = 0.0f, const float range_max = 1.0f );

#define CACHE_LINE_SIZE 64

/*! \struct CellIndices
\brief Struktura adresy buky v rmci celho objemu.

\author Tom Fabin
\version 1.0
\date 2015-2020
*/
struct CellIndices
{
public:
	CellIndices()
	{
		i = -1;
		j = -1;
		k = -1;
	}

	CellIndices( const int i, const int j, const int k );

	int i;
	int j;
	int k;
};

/*! \struct CellHit
\brief Struktura popisujc zasaenou buku (jej index v rmci celho objemu a parametrick body vstupu a vstupu paprsku).

\author Tom Fabin
\version 1.0
\date 2015-2020
*/
struct CellHit
{
public:
	CellHit( CellIndices & indices, const float t_0, const float t_1 )
	{
		assert( ( t_0 >= 0.0f ) && ( t_1 >= t_0 ) );
		// poku je t_0 == t_1, pak to prochz jen rohem buky a ta me bt peskoena

		this->indices = indices;
		this->t_0 = t_0;
		this->t_1 = t_1;

		f = 0.0f;
	}

	CellIndices indices;

	float t_0;
	float t_1;

	float f;
};

/*! \class Volume
\brief Tda pro pmou vizualizaci skalrnch objemovch dat.

\author Tom Fabin
\version 1.0
\date 2015-2020
*/
class Volume
{
public:
	Volume( const int width, const int height, const int n, const Vector3 & cell_size );
	~Volume();

	void Load( std::string & file_name_mask, const int first_slice_index, const int last_slice_index );
	
	void Generate();		

	Cell cell( const CellIndices & indices ) const;
	Cell cell( const int i, const int j, const int k ) const;

	CellIndices cell_indices( const Vector3 & p ) const; // vrt indexy buky, ve kter le bod p
	
	void Traverse( Ray & ray, std::vector<CellHit> & traversed_cells );

	void Raycast( Camera & camera, const int samples = 1 );
	
protected:
	int offset( const int i, const int j, const int k ) const;	

private:
	int width_; // horizontln rozlien zdrojovho snmku [px]; osa y
	int height_; // vertikln rozlien zdrojovho snmku [px]; osa -z
	int n_; // poet ez [-]; osa x

	int width_step_; // poet relnch hodnot na jednom dku datov matice [-]
	
	Vector3 cell_size_; // rozmry buky [m]
	Vector3 half_volume_size_; // rozmry plky celho objemu [m]

	DATA_TYPE * data_; // datov matice s hustotami o rozmrech (width_step_, height_, n_) a v tomto poad jsou tak uloeny jednotliv hodnoty

	AABB bounds_;

	static const int kElement_size = sizeof( DATA_TYPE );
};

#endif
